import java.io.*;
import java.util.*;
import javax.sound.sampled.*;

/**
 * Representation of a single audio chunk (i.e. a single line of an
 * EDL or feature file).
 *
 * @author Ron Weiss (ronw@ee.columbia.edu)
 */

public class Chunk implements Comparable
{
    String srcFile;
    double startTime;
    double length;
    
    String comment = "";

    /**
     * Chunk constructor
     */
    public Chunk(String sf, double st, double l)
    {
        srcFile = sf;
        startTime = st;
        length = l;
    }
    
    /**
     * Load samples corresponding to this Chunk
     */
    public double[] getSamples(AudioFormat format) throws IOException
    {
        AudioInputStream ais = MEAPUtil.openInputStream(srcFile, format);
        
        // this format had better match the output file format or we
        // have problems
        AudioFormat fmt = ais.getFormat();
        
        int frameSize = fmt.getFrameSize();
        float frameRate = fmt.getFrameRate();
        
        int startByte = (int)(startTime*frameRate*frameSize);
        int byteLen = (int)(length*frameRate*frameSize);
        
        byte[] bytes = new byte[byteLen];
        ais.skip(startByte);
        ais.read(bytes, 0, byteLen);
        ais.close();
        
        double[] samples = new double[(int)(frameRate*length)];
        MEAPUtil.bytes2doubles(bytes, samples, fmt);
        
//         // Smooth out any rough edges to the data with a simple triangular
//         // window on each end
//         for(int i=0; i<winSize; i++) 
//         {
//             samples[i] *= (double)i / (double)winSize;
//             samples[samples.length-1-i] *= (double)i / (double)winSize;
//         }    
        
        return samples;
    }
    
    /**
     * Compare one Chunk to another - comparisons are
     * performed based on startTime field
     */
    public int compareTo(Object o) throws ClassCastException
    {
        Chunk c = (Chunk)o;
        return Double.compare(startTime, c.startTime);
    }

    public String toString()
    {
        // concatenating strings is super slow.  Better to use StringBuffer
        // guesstimate the string length
        StringBuffer s = new StringBuffer(200);
        s.append(srcFile).append(" ").append(startTime).append(" ");
        s.append(length).append(" ").append(comment).append("\n");
        
        return s.toString();
    }
}

