import java.util.*;
/**
 * Representation of an EDLFile Chunk
 *
 * @author Ron Weiss (ronw@ee.columbia.edu)
 */

public class EDLChunk extends Chunk
{
    double dstTime;
    Vector commands; 
 
    /**
     * EDLFile Chunk constructor
     */
    public EDLChunk(String sf, double st, double l, double dt)
    {
        super(sf,st,l);
        dstTime = dt;

        commands = new Vector();
    }
   
    public EDLChunk(Chunk c, double dt)
    {
        this(c.srcFile, c.startTime, c.length, dt);
    }

    /**
     * Compare one EDLChunk to another - comparisons are
     * performed based on dstTime field
     */
    public int compareTo(Object o) throws ClassCastException
    {
        try
        {
            EDLChunk c = (EDLChunk)o;
            return Double.compare(dstTime, c.dstTime);
        }
        catch(ClassCastException e)
        {
            return super.compareTo(o);
        }
    }

    /**
     * Write a description of this Chunk as a String in the format
     * expected in an EDLFile
     */
    public String toString()
    {
        StringBuffer s = new StringBuffer(200);
        s.append(dstTime).append(" ").append(srcFile).append(" ");
        s.append(startTime).append(" ").append(length).append(" ");

        if(commands != null)
        {
            Iterator x = commands.iterator();
            while(x.hasNext())
                s.append(x.next()).append(" ");
        }

        s.append(comment).append("\n");
        
        return s.toString();
    }
}

