import java.util.*;

// I can't believe that the huge Java API doesn't already include such
// a basic data structure

/**
 * Implementation of a binary max heap.
 *
 * @author Ron Weiss (ronw@ee.columbia.edu)
 */
public class MaxHeap extends Heap
{
    public MaxHeap()
    {
        super();
    }

    /**
     *  Use given Comparator for all comparisons between elements in
     *  this Heap.  Otherwise rely on compareTo methods and Comparable
     *  Objects.
     */
    public MaxHeap(Comparator c)
    {
        super(c);
    }

    public MaxHeap(int capacity)
    {
        super(capacity);
    }

    public MaxHeap(Collection c)
    {
        super(c);
    }

    public Object deleteMax()
    {
        return remove(0);
    }

    /**
     * Compare two Objects in this heap - wrapper around
     * compareTo/Comparator.compare
     */
    protected int cmp(int node1, int node2)
    {
        return -super.cmp(node1, node2);
    }

    // Simple test program
    public static void main(String args[])
    {
        MaxHeap h = new MaxHeap();
        Vector v = new Vector();

        //int[] numbers = {10, 1, 6, -10, 2, 7, 6};
        int[] numbers = {10, 1, 6, -10, 2, 7, 4};

        try
        {
            System.out.print("Adding ");
            Double d = null;
            for(int x = 0; x < numbers.length; x++)
            {
                System.out.print(numbers[x] + " ");
                d = new Double(numbers[x]);
                h.add(d);
                v.add(d);
            }

            System.out.print("\nCalling deleteMax: ");
            for(int x = 0; x < numbers.length; x++)
                System.out.print(h.deleteMax() + " ");

            System.out.print("\nRemoving an element (this should obey the heap property): ");
            h.addAll(v);
            h.remove(d);
            Iterator i = h.iterator();
            while(i.hasNext())
                System.out.print(i.next() + " ");

            System.out.print("\nRunning heapSort: ");
            //h.addAll(v);
            h.sort();
            i = h.iterator();
            while(i.hasNext())
                System.out.print(i.next() + " ");
            System.out.print("\n");
        }
        catch(Exception e)
        {
            e.printStackTrace();
        }
    }
}
