import java.util.*;

/**
 * Implementation of a binary min heap
 *
 * @author Ron Weiss (ronw@ee.columbia.edu)
 */
public class MinHeap extends Heap
{
    public MinHeap()
    {
        super();
    }

    /**
     *  Use given Comparator for all comparisons between elements in
     *  this Heap.  Otherwise rely on compareTo methods and Comparable
     *  Objects.
     */
    public MinHeap(Comparator c)
    {
        super(c);
    }

    public MinHeap(int capacity)
    {
        super(capacity);
    }

    public MinHeap(Collection c)
    {
        super(c);
    }

    public Object deleteMin()
    {
        return remove(0);
    }

    /**
     * Perform heap sort on the data stored in this heap.  After
     * calling sort, a call to this objects iterator() method will
     * iterate through the data stored in the heap in sorted order.
     */
    public void sort()
    {
        super.sort();

        // reverse the vector since we want to sort in ascending order
        for(int x = 0; x <= (size()-1)/2; x++)
        {
            Object tmp = get(x);
            set(x, get(size()-1-x));
            set(size()-1-x, tmp);
        }
    }

    // Simple test program
    public static void main(String args[])
    {
        MinHeap h = new MinHeap();
        Vector v = new Vector();

        //int[] numbers = {10, 1, 6, -10, 2, 7, 6};
        int[] numbers = {10, 1, 6, -10, 2, 7, 4};

        try
        {
            System.out.print("Adding ");
            Double d = null;
            for(int x = 0; x < numbers.length; x++)
            {
                System.out.print(numbers[x] + " ");
                d = new Double(numbers[x]);
                h.add(d);
                v.add(d);
            }

            System.out.print("\nCalling deleteMin: ");
            for(int x = 0; x < numbers.length; x++)
                System.out.print(h.deleteMin() + " ");

            System.out.print("\nRemoving an element (this should obey the heap property): ");
            h.addAll(v);
            h.remove(d);
            Iterator i = h.iterator();
            while(i.hasNext())
                System.out.print(i.next() + " ");

            System.out.print("\nRunning heapSort: ");
            //h.addAll(v);
            h.sort();
            i = h.iterator();
            while(i.hasNext())
                System.out.print(i.next() + " ");
            System.out.print("\n");
        }
        catch(Exception e)
        {
            e.printStackTrace();
        }
    }
}
