package featextractors;

/**
 * A simple feature calculation that averages the spectral centroid of
 * all spectral frames in a given chunk.
 *
 * @author Ron Weiss (ronw@ee.columbia.edu)
 */

public class AvgSpecCentroid implements FeatureExtractor 
{
    public double[] features(STFT stft, long startFrame, int length) 
    {
        double[] curFrame;
        double[] avgSpecCentroid = new double[1];
        double num = 0;
        double den = 0;

        for(int frame=0; frame<length; frame++) 
        {
            num = 0;  den = 0;
            curFrame = stft.getFrame(startFrame+frame);
            for(int band=0; band<stft.getRows(); band++) 
            {
                double freqCenter = band*(stft.samplingRate/2)/(stft.getRows()-1);
                // convert back to linear power
                double p = Math.pow(10,curFrame[band]/10);

                num += freqCenter*p;
                den += p;
            }
            avgSpecCentroid[0] += num/(length*den);
        }
        
        return avgSpecCentroid;
    }
}
