package featextractors;

/**
 * A simple feature calculation that averages the spectral flatness of
 * all spectral frames in a given chunk.
 *
 * @author Ron Weiss (ronw@ee.columbia.edu)
 */

public class AvgSpecFlatness implements FeatureExtractor 
{
    public double[] features(STFT stft, long startFrame, int length) 
    {
        double[] curFrame;
        double[] avgSpecFlatness= new double[1];
        double num = 1;
        double den = Double.MIN_VALUE;

        avgSpecFlatness[0] = 0;

        for(int frame=0; frame<length; frame++) 
        {
            num = 0; den = 0;
            int nband = stft.getRows();
            curFrame = stft.getFrame(startFrame+frame);
            for(int band = 0; band < nband; band++) 
            {
                // convert back to linear power
                double p = Math.pow(10,curFrame[band]/10);

                num += Math.log(p)/nband;
                den += p/nband;
            }
            avgSpecFlatness[0] += Math.exp(num)/(den*length);
        }
        
        return avgSpecFlatness;
    }
}
