package featextractors;

/**
 * A simple feature calculation that computes the centroid in time and
 * frequency of the STFT time/frequency surface of a given chunk.
 *
 * @author Ron Weiss (ronw@ee.columbia.edu)
 */

public class TimeFreqCentroid implements FeatureExtractor 
{
    public double[] features(STFT stft, long startFrame, int length) 
    {
        double[] curFrame;
        double[] TimeFreqCentroid = new double[2];
        double num0 = 0;
        double num1 = 0;
        double den = 0;

        for(int frame=0; frame<length; frame++) 
        {
            curFrame = stft.getFrame(startFrame+frame);
            double timeCenter = stft.fr2Seconds(frame);

            for(int band=0; band<stft.getRows(); band++) 
            {
                double freqCenter = band*(stft.samplingRate/2)/(stft.getRows()-1);
                // convert back to linear power
                double p = Math.pow(10,curFrame[band]/10);

                num0 += timeCenter*p;
                num1 += freqCenter*p;
                den += p;
            }
        }
        
        TimeFreqCentroid[0] = num0/den;
        TimeFreqCentroid[1] = num1/den;

        return TimeFreqCentroid;
    }
}
